import datetime
import uuid

from datetime import datetime, timedelta
from battlemetrics.components.helpers import Helpers

class Organization:
    def __init__(self, helpers: Helpers, base_url: str) -> None:
        self.helpers = helpers
        self.base_url = base_url


    async def info(self, organization_id: int) -> dict:
        """Returns an organizations profile.
        Documentation: Not documented in the API.
        Args:
            organization_id (int): An organizations battlemetrics ID.
        Returns:
            dict: The information about your organization or a targeted organization
        """

        url = f"{self.base_url}/organizations/{organization_id}"
        data = {
            "include": "organizationUser,banList,role,organizationStats"
        }
        return await self.helpers._make_request(method="GET", url=url, params=data)

    async def stats(self, organization_id: int, start: str, end: str, game: str = None) -> dict:
        """Gets the player stats for the organization
        Documentation: https://www.battlemetrics.com/developers/documentation#resource-organizationStats
        Args:
            organization_id (int): Organization ID
            start (str): UTC start time. Defaults to 7 days ago.
            end (str): UTC end time. Defaults to today.
            game (str, optional): Targeted game, example: rust. Defaults to None.
        Returns:
            dict: Player stats for the organization.
        """

        if not start:
            now = datetime.utcnow()
            start = now - timedelta(days=1)
            start = start.strftime('%Y-%m-%dT%H:%M:%SZ')
        if not end:
            end = datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%SZ')
        url = f"{self.base_url}/organizations/{organization_id}/stats/players"
        data = {
            "filter[range]": f"{start}:{end}"
        }
        if game:
            data["filter[game]"] = game
        return await self.helpers._make_request(method="GET", url=url, params=data)

    async def friends_list(self, organization_id: str, filter_accepted: bool = True, filter_origin: bool = True, filter_name: str = None, filter_reciprocated: bool = True) -> dict:
        """Gets all the organization friends.
        Documentation: https://www.battlemetrics.com/developers/documentation#resource-organizationFriend
        Args:
            organization_id (str): Your organization ID
            filter_accepted (bool, optional): True or False. Have they accepted our friendship?. Defaults to True.
            filter_origin (bool, optional): True or False. Defaults to True.
            filter_name (str, optional): Name of a specific organization. Defaults to None.
            filter_reciprocated (bool, optional): True or False. Are the feelings mutual?. Defaults to True.
        Returns:
            dict: Returns all the friendship information based on the paramaters set.
        """

        url = f"{self.base_url}/organizations/{organization_id}/relationships/friends"
        data = {
            "include": "organization",
            "filter[accepted]": str(filter_accepted).lower(),
            "filter[origin]": str(filter_origin).lower(),
            "filter[reciprocated]": str(filter_reciprocated).lower()
        }
        if filter_name:
            data['filter[name]'] = filter_name
        return await self.helpers._make_request(method="GET", url=url, params=data)

    async def friend(self, organization_id: int, friend_organization_id: int) -> dict:
        """Gets the friend information for your organization.
        Documentation: https://www.battlemetrics.com/developers/documentation#link-GET-organizationFriend-/organizations/{(%23%2Fdefinitions%2Forganization%2Fdefinitions%2Fidentity)}/relationships/friends/{(%23%2Fdefinitions%2Forganization%2Fdefinitions%2Fidentity)}
        Args:
            organization_id (int): Your organization ID
            friend_organization_id (int): Friend organization ID
        Returns:
            dict: Dictionary response about the organization friendship
        """

        url = f"{self.base_url}/organizations/{organization_id}/relationships/friends/{friend_organization_id}"
        data = {
            "include": "organization,playerFlag,organizationStats"
        }
        return await self.helpers._make_request(method="GET", url=url, params=data)

    async def friend_update(self, organization_id: int, friend_organization_id: int, identifiers: list, playerflag: str, shared_notes: bool = True, accepted: bool = True) -> dict:
        """Updates your organizations friendship.
        Documentation: https://www.battlemetrics.com/developers/documentation#link-PATCH-organizationFriend-/organizations/{(%23%2Fdefinitions%2Forganization%2Fdefinitions%2Fidentity)}/relationships/friends/{(%23%2Fdefinitions%2Forganization%2Fdefinitions%2Fidentity)}
        Args:
            organization_id (int): Your organization ID
            friend_organization_id (int): The friendly organizations ID.
            identifiers (list): [ip, steamID], identifiers to be shared.
            shared_notes (bool, optional): Sharing Notes?
            accepted (bool, optional):Accepted friendship?
        Returns:
            dict: Returns a dictionary response on the new updated friendship.
        """

        url = f"https://api.battlemetrics.com/organizations/{organization_id}/relationships/friends/{friend_organization_id}"
        data = {
            "data": {
                "id": friend_organization_id,
                "type": "organizationFriend",
                "attributes": {
                    "accepted": str(accepted).lower(),
                    "identifiers": identifiers,
                    "notes": str(shared_notes).lower()
                }
            }
        }
        return await self.helpers._make_request(method="PATCH", url=url, json_dict=data)

    async def friend_create(self, organization_id: int, friendly_org: int, identifiers: list, shared_notes: bool = True) -> dict:
        """Creates a new friend invite to the targeted organization ID
        Documentation: https://www.battlemetrics.com/developers/documentation#link-POST-organizationFriend-/organizations/{(%23%2Fdefinitions%2Forganization%2Fdefinitions%2Fidentity)}/relationships/friends
        Args:
            organization_id (int): Your organization ID
            friendly_org (int): Targeted organization ID
            identifiers (list): ["steamID", "ip"]
            shared_notes (bool, optional): Sharing notes? Defaults to True.
        Returns:
            dict: Returns the dictionary response from the server
        """

        url = f"https://api.battlemetrics.com/organizations/{organization_id}/relationships/friends"
        data = {
            "data": {
                "type": "organizationFriend",
                "attributes": {
                    "identifiers": identifiers,
                    "notes": str(shared_notes).lower()
                },
                "relationships": {
                    "friend": {
                        "data": {
                            "type": "organization",
                            "id": f"{friendly_org}"
                        }
                    },
                    "flagsShared": {
                        "data": [
                            {
                                "type": "playerFlag",
                                "id": f"{uuid.uuid5}"
                            }
                        ]
                    }
                }
            }
        }
        return await self.helpers._make_request(method="POST", url=url, json_dict=data)

    async def friend_delete(self, organization_id: int, friends_id: int) -> dict:
        """Deletes a friendship
        Documentation: https://www.battlemetrics.com/developers/documentation#link-DELETE-organizationFriend-/organizations/{(%23%2Fdefinitions%2Forganization%2Fdefinitions%2Fidentity)}/relationships/friends/{(%23%2Fdefinitions%2Forganization%2Fdefinitions%2Fidentity)}
        Args:
            organization_id (int): Your organization ID
            friends_id (int): Friends organization ID
        Returns:
            dict: Response from the server.
        """

        url = f"{self.base_url}/organizations/{organization_id}/relationships/friends/{friends_id}"
        return await self.helpers._make_request(method="DELETE", url=url)

    async def player_stats(self, organization_id: int, start_date: str = None, end_date: str = None, game:str = None) -> dict:
        """Returns the statistics of all the players who have joined your server and where they're from.
        Documentation: https://www.battlemetrics.com/developers/documentation#link-GET-organization-/organizations/{(%23%2Fdefinitions%2Forganization%2Fdefinitions%2Fidentity)}/stats/players
        Args:
            organization_id (int): Your organization ID
            start_date (str, optional): Start date, max 90 days. Defaults to 90 days ago.
            end_date (str, optional): End date, defaults to now.
            game (str, optional): The game you wish to filter by. Defaults to None
        Returns:
            dict: Returns a dictionary of all the stats!
        """

        url = f"{self.base_url}/organizations/{organization_id}/stats/players"
        if not start_date:
            now = datetime.utcnow()
            start_date = now - timedelta(days=1)
            start_date = start_date.strftime('%Y-%m-%dT%H:%M:%SZ')
        if not end_date:
            end_date = datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%SZ')
        data = {
            
            "filter[range]": f"{start_date}:{end_date}"
        }
        
        if game:
            data["filter[game]"] = game
        
        return await self.helpers._make_request(method="GET", url=url, params=data)

    async def commands_activity(self, organization_id: int, summary: bool = False, users: str = None, commands: str = None, time_start: str = None, time_end: str = None, servers: int = None) -> dict:
        """Grabs all the command activity related to the targeted organization
        Documentation: https://www.battlemetrics.com/developers/documentation#link-GET-commandStats-/organizations/{(%23%2Fdefinitions%2Forganization%2Fdefinitions%2Fidentity)}/relationships/command-stats
        Args:
            organization_id (int): The Organization ID
            summary (bool, optional): A summary. Defaults to False.
            users (str, optional): Specific users?. Defaults to None.
            commands (str, optional): Specific Commands?. Defaults to None.
            time_start (str, optional): UTC start time. Defaults to 7 days ago.
            time_end (str, optional): UTC end time. Defaults to today.
            servers (int, optional): Targeted servers. Defaults to None.
        Returns:
            dict: Returns command usage data.
        """

        if not time_start:
            now = datetime.utcnow()
            time_start = now - timedelta(days=1)
            time_start = time_start.strftime('%Y-%m-%dT%H:%M:%SZ')
        if not time_end:
            time_end = datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%SZ')
        data = {
            "filter[timestamp]": f"{time_start}:{time_end}"
        }
        if summary:
            data['filter[summary]'] = str(summary).lower()
        if users:
            data['filter[users]'] = users
        if commands:
            data['filter[commands]'] = commands
        if servers:
            data['filter[servers]'] = servers
        url = f"{self.base_url}/organizations/{organization_id}/relationships/command-stats"
        return await self.helpers._make_request(method="GET", url=url, params=data)

    async def user_organization_view(self) -> dict:
        """Retrieves the organizations the current API token can view.
        Documentation: This endpoint is not documented.

        Returns:
            dict: Returns a dictionary of all the organizations the user can view.
        """
        url = f"{self.base_url}/organizations"
        data = {
            "page[size]": "100",
            "include": "organizationUser,banList,organizationStats"
        }

        return await self.helpers._make_request(method="GET", url=url, params=data)



from datetime import datetime, timedelta

#Components
from battlemetrics.components.banlist import BanList
from battlemetrics.components.bans import Bans
from battlemetrics.components.flags import Flags 
from battlemetrics.components.gameinfo import GameInfo
from battlemetrics.components.helpers import Helpers
from battlemetrics.components.notes import Notes
from battlemetrics.components.organization import Organization
from battlemetrics.components.player import Player
from battlemetrics.components.server import Server
from battlemetrics.components.session import Session
    
class Battlemetrics:
    def __init__(self, api_key: str) -> None:
        self.base_url = "https://api.battlemetrics.com"
        self.api_key = api_key
    
    @property
    def helpers(self) -> Helpers:
        return Helpers(api_key=self.api_key)
    
    @property
    def player(self) -> Player:
        return Player(helpers=self.helpers, base_url=self.base_url)
    
    @property
    def server(self) -> Server:
        return Server(helpers=self.helpers, base_url=self.base_url)
    
    @property
    def notes(self) -> Notes:
        return Notes(helpers=self.helpers, base_url=self.base_url)
    
    @property
    def flags(self) -> Flags:
        return Flags(helpers=self.helpers, base_url=self.base_url)
    
    @property
    def session(self) -> Session:
        return Session(helpers=self.helpers, base_url=self.base_url)
    
    @property
    def banlist(self) -> BanList:
        return BanList(helpers=self.helpers, base_url=self.base_url)
    
    @property
    def organization(self) -> Organization:
        return Organization(helpers=self.helpers, base_url=self.base_url)
    
    @property
    def gameinfo(self) -> GameInfo:
        return GameInfo(helpers=self.helpers, base_url=self.base_url)
    
    @property
    def bans(self) -> Bans:
        return Bans(helpers=self.helpers, base_url=self.base_url)

    def check_api_scopes(self, token: str = None) -> dict:
        """Retrieves the tokens scopes from the oauth.
        Documentation: None.
        Args:
            api_key (str, optional): Your given API token. Defaults to the one supplied to this battlemetrics class.
        Returns:
            dict: The tokens data.
        """
        
        if not token:
            token = self.api_key

        url = f"https://www.battlemetrics.com/oauth/introspect"
        data = {
            "token": token
        }
        return self.helpers._make_request(method="POST", url=url, json_dict=data)

    def metrics(self, name: str = "games.rust.players", start_date: str = None, end_date: str = None, resolution: str = "60") -> dict:
        """A data point as used in time series information.
        Documentation: https://www.battlemetrics.com/developers/documentation#link-GET-dataPoint-/metrics
        Args:
            name (str, optional): "games.{game}.players" and "games.{game}.players.steam", defaults to "games.rust.players"
            start_date (str, optional) UTC time format. Defaults to Current Date.
            end_date (str, optional): UTC time format. Defaults to 1 day ago.
            resolution (str, optional): raw, 30, 60 or 1440. Defaults to "60".
        Returns:
            dict: a bunch of numbers.
        """

        url = f"{self.base_url}/metrics"
        
        if not start_date:
            now = datetime.utcnow()
            start_date = now - timedelta(days=1)
            start_date = start_date.strftime('%Y-%m-%dT%H:%M:%SZ')
        if not end_date:
            end_date = datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%SZ')
        data = {
            "metrics[0][name]": name,
            "metrics[0][range]": f"{start_date}:{end_date}",
            "metrics[0][resolution]": resolution,
            "fields[dataPoint]": "name,group,timestamp,value"
        }
        return self.helpers._make_request(method="GET", url=url, params=data)

    def activity_logs(self, filter_bmid: int = None, filter_search: str = None, filter_servers: int = None, blacklist: str = None, whitelist: str = None) -> dict:
        """Retrieves the activity logs.

        Args:
            filter_bmid (int, optional): A battlemetrics ID for a specific user. Defaults to None.
            filter_search (str, optional): What do you want to search?. Defaults to None.
            filter_servers (int, optional): A specific battlemetrics server ID. Defaults to None.
            blacklist (str, optional): Example: unknown, playerMessage. Defaults to None.
            whitelist (str, optional): unknown, playerMessage. Defaults to None.

        Returns:
            dict: The activity logs information.
        """

        url = f"{self.base_url}/activity"
        data = {
            "page[size]": "100",
            "include": "organization,server,user,player"
        }

        if blacklist:
            data['filter[types][blacklist]'] = blacklist
        if whitelist:
            data['filter[types][whitelist]'] = whitelist
        if filter_servers:
            data['filter[servers]'] = filter_servers
        if filter_search:
            data['filter[search]'] = filter_search
        if filter_bmid:
            data['filter[players]'] = filter_bmid
        
        return self.helpers._make_request(method="GET", url=url, params=data)


import datetime

from datetime import datetime, timedelta
from battlemetrics.components.helpers import Helpers

class Server:
    def __init__(self, base_url: str, helpers: Helpers) -> None:
        self.base_url = base_url
        self.helpers = helpers

    async def leaderboard_info(self, server_id: int,  start: str = None, end: str = None, player: int = None) -> dict:
        """Displays the leaderboard for a specific player.
        Documentation: https://www.battlemetrics.com/developers/documentation#link-GET-leaderboardPlayer-/servers/{(%23%2Fdefinitions%2Fserver%2Fdefinitions%2Fidentity)}/relationships/leaderboards/time
        Args:
            server_id (int): The server ID
            player (int): Battlemetrics player ID
            start (str): UTC Start date. Defaults to 1 day ago.
            end (str): UTC End date. Defaults to today.
        Returns:
            dict: Returns the leaderboard information for the player.
        """

        if not start:
            now = datetime.utcnow()
            start = now - timedelta(days=1)
            start = start.strftime('%Y-%m-%dT%H:%M:%SZ')
        if not end:
            end = datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%SZ')
        data = {
            "page[size]": "100",
            "filter[period]": f"{start}:{end}",
            "fields[leaderboardPlayer]": "name,value"
        }
        if player:
            data['filter[player]'] = player
        url = f"{self.base_url}/servers/{server_id}/relationships/leaderboards/time"
        return await self.helpers._make_request(method="GET", url=url, params=data)
    
    async def search(self,*,
                     search:str = None,
                     countries:list[str]=None,
                     game: str = None,
                     blacklist: list[str] = None,
                     whitelist: list[str] = None,
                     organization: str = None,
                     rcon: bool = True,
                     server_type:list[str] = None,
                     game_mode:list[str]=None,
                     gather_rate_min:int=None,
                     gather_rate_max:int=None,
                     group_size_min:int=None,
                     group_size_max:int=None,
                     map_size_min:int=None,
                     map_size_max:int=None,
                     blueprints:str="both",
                     pve:str="both",
                     kits:str="both",
                     status:bool=True) -> dict:
        
        """List, search and filter servers.
        Documentation: https://www.battlemetrics.com/developers/documentation#link-GET-server-/servers
        Args:
            search (str, optional): Search for specific server. Defaults to None.
            countries (list, optional): Server in a country. Defaults to None.
            game (str, optional): Specific game. Defaults to None.
            blacklist (str, optional): Blacklisted servers (comma seperated). Defaults to None.
            whitelist (str, optional): Whitelisted servers (comma seperated). Defaults to None.
            organization (str, optional): Organization ID. Defaults to None.
            rcon (bool, optional): RCON only. Defaults to False.
            server_type (list, optional): takes a list with any of the following: official, modded, community. Defaults to None,
            game_mode (list, optional): standard or hardcore. Defaults to none.
            gather_rate_min (int, optional): 1 or none.
            gather_rate_max (int, optional): Max 9999999
            group_size_min (int, optional): 1 or none.
            group_size_max (int, optional): Max 255
            map_size_min (int, optional): 1 or none.
            map_size_max (int, optional): Max 9999999
            blueprints (string): Takes 1 of 3 options: True, False or Both. Defaults to both.
            pvp (string): Takes 1 of 3 options: True, False or Both. Defaults to both.
            kits (string): Takes 1 of 3 options: True, False or Both. Defaults to both.
            
        Returns:
            dict: Dictionary response from battlemetrics.
        """

        url = f"{self.base_url}/servers"
        
        server_type_uuid = "845b5e50-648f-11ea-aa7c-b3870f9c01b3"
        server_types = {
            "official": "689d22c5-66f4-11ea-8764-b7f50ac8fe2a",
            "community": "689d22c6-66f4-11ea-8764-e75bf88ce534",
            "modded": "689d22c4-66f4-11ea-8764-ff40d927c47a"
        }
        
        #New Stuff to add! \o/
        blueprints_uuid = "ce84a17d-a52b-11ee-a465-1798067d9f03" #Boolean
        pve_uuid = "689d22c2-66f4-11ea-8764-e7fb71d2bf20" #boolean
        kits_uuid = "ce84a17c-a52b-11ee-a465-1fcfab67c57a" #Boolean
        
        
        
        #Gamemode
        gamemode_uuid = "796542ee-36ed-11ed-873e-631bb6c8148e" #List, standard or hardcore.
        game_modes = {
            "standard": "7eaae984-36ed-11ed-873e-9b4d5140c855",
            "hardcore": "7eaae985-36ed-11ed-873e-4f5345affee4" 
        }
        
        
        mapsize_uuid = "689d22c3-66f4-11ea-8764-5723d5d7cfba" #1:9999999
        grouplimit_uuid = "ce84a17e-a52b-11ee-a465-a3c586d9e374" #1:255
        gatherrate_uuid = "ce84a17f-a52b-11ee-a465-33d2d6d4f5ea" #1:255

    
        data = {}
        data['page[size]'] = "100"
        data['include'] = "serverGroup"
        data['filter[rcon]'] = str(rcon).lower()
        
        if not status:
            data['filter[status]'] = "offline,dead,invalid"
        else:
            data['filter[status]'] = "true"
            
        if search:
            data["filter[search]"] = search
        if countries:
            data["filter[countries]"] = countries
        if game:
            data["filter[game]"] = game
        if blacklist:
            data["filter[ids][blacklist]"] = blacklist
        if whitelist:
            data["filter[ids][whitelist]"] = whitelist
        if organization:
            data["filter[organizations]"] = int(organization)
            
        if group_size_min or group_size_max:
            data[f"filter[features][{grouplimit_uuid}]"] = f"{group_size_min}:{group_size_max}"
            
        if map_size_min or map_size_max:
            data[f"filter[features][{mapsize_uuid}]"] = f"{map_size_min}:{map_size_max}"
            
        if gather_rate_min or gather_rate_max:
            data[f"filter[features][{gatherrate_uuid}]"] = f"{gather_rate_min}:{gather_rate_max}"
            
        if blueprints.lower() == "true" or blueprints.lower() == "false":
            data[f"filter[features][{blueprints_uuid}]"] = f"{blueprints}"
            
        if pve.lower() == "true" or pve.lower() == "false":
            data[f"filter[features][{pve_uuid}]"] = f"{pve}"
            
        if kits.lower() == "true" or kits.lower() == "false":
            data[f"filter[features][{kits_uuid}]"] = f"{kits}"
        
          
        if server_type:
            count = 0
            features = None
            for ServerType in server_type:
                if features:
                    features += f"&filter[features][{server_type_uuid}][or][{count}]={server_types[ServerType.lower]}"
                else:
                    features = f"filter[features][{server_type_uuid}][or][{count}]={server_types[ServerType.lower]}"
                count += 1
            if features:
                url += f"?{features}"
        
        if game_mode:
            count = 0
            features = None
            for mode in game_mode:
                if features:
                    features += f"&filter[features][{gamemode_uuid}][or][{count}]={game_modes[mode.lower]}"
                else:
                    features = f"filter[features][{gamemode_uuid}][or][{count}]={game_modes[mode.lower]}"
                count += 1
            if features:
                url += f"?{features}"
        return await self.helpers._make_request(method="GET", url=url, params=data)
    
    async def create(self, server_ip: str, server_port: str, port_query: str, game: str, server_gsp: str = None, organization_id: int = None, banlist_id: str = None, server_group: str = None) -> dict:
        """Add a server to the system.
        Documentation: https://www.battlemetrics.com/developers/documentation#link-POST-server-/servers
        The documentation does not provide information on how to properly use the params after the "Game" param.
        Args:
            server_ip (str): The IP of your server
            server_port (str): The port of the server
            port_query (str): The port query of the server
            game (str): The game of the server
            server_gsp (str): game server provider
            organization_id (int): The organization ID the server belongs to
            banlist_id (str): A banlist ID the server uses.
            server_group (str): The server group.
        Returns:
            dict: Response from battlemetrics.
        """

        url = f"{self.base_url}/servers"
        data = {
            "data": {
                "type": "server",
                "attributes": {
                    "ip": f"{server_ip}",
                    "port": f"{server_port}",
                    "portQuery": f"{port_query}"
                },
                "relationships": {
                    "game": {
                        "data": {
                            "type": "game",
                            "id": f"{game}"
                        }
                    }
                }
            }
        }
        return await self.helpers._make_request(method="POST", url=url, json_dict=data)

    async def enable_rcon(self, server_id: int) -> dict:
        # This endpoint is not completed by the creator of this wrapper.
        print("This endpoint is not completed by the creator of this wrapper.")
        pass

    async def console_command(self, server_id: int, command: str) -> dict:
        """
        Sends a raw server console command. These commands are usually what you can type in game via the F1 console.
        An example: mute <steamid> <duration> <reason>
        another is: kick <steamid>
        Args:
            server_id (int): The server you want the command to run on.
            command (str): The command you want to attempt to run!
        Returns:
            dict: If it was successful or not.
        """

        url = f"{self.base_url}/servers/{server_id}/command"
        data = {
            "data":
            {
                "type": "rconCommand",
                "attributes":
                {
                    "command": "raw",
                    "options":
                    {
                        "raw": f"{command}"
                    }
                }
            }
        }
        return await self.helpers._make_request(method="POST", url=url, json_dict=data)

    async def send_chat(self, server_id: int, message: str, sender_name: str) -> dict:
        """
        Sends a raw server console command. These commands are usually what you can type in game via the F1 console.
        An example: mute <steamid> <duration> <reason>
        another is: kick <steamid>
        Args:
            server_id (int): The server you want the command to run on.
            message (str): The message you wish to send
            sender_name (str): Who do you want to send the message as?
        Returns:
            dict: If it was successful or not.
        """

        url = f"{self.base_url}/servers/{server_id}/command"
        chat = {
            "data": {
                "type": "rconCommand",
                "attributes": {
                        "command": "rust:globalChat",
                        "options": {
                            "message": f"{sender_name}: {message}"
                        }
                }
            }
        }
        return await self.helpers._make_request(method="POST", url=url, json=chat)

    async def delete_rcon(self, server_id: int) -> dict:
        """
        Names on the tin, deletes the RCON for your server
        Documentation: https://www.battlemetrics.com/developers/documentation#link-DELETE-server-/servers/{(%23%2Fdefinitions%2Fserver%2Fdefinitions%2Fidentity)}/rcon
        Args:
            server_id (int): The server ID.
        Returns:
            dict: Response from the server.
        """

        url = f"{self.base_url}/servers/{server_id}/rcon"
        return await self.helpers._make_request(method="DELETE", url=url)

    async def disconnect_rcon(self, server_id: int) -> dict:
        """
        Names on the tin, disconnects RCON from your server.
        Documentation: https://www.battlemetrics.com/developers/documentation#link-DELETE-server-/servers/{(%23%2Fdefinitions%2Fserver%2Fdefinitions%2Fidentity)}/rcon/disconnect
        Args:
            server_id (int): Server ID
        Returns:
            dict: Response from the server.
        """

        url = f"{self.base_url}/servers/{server_id}/rcon/disconnect"
        return await self.helpers._make_request(method="DELETE", url=url)

    async def connect_rcon(self, server_id: int) -> dict:
        """Names on the tin, connects RCON to your server.
        Documentation: https://www.battlemetrics.com/developers/documentation#link-DELETE-server-/servers/{(%23%2Fdefinitions%2Fserver%2Fdefinitions%2Fidentity)}/rcon/connect
        Args:
            server_id (int): Server ID
        Returns:
            dict: Response from the server.
        """

        url = f"{self.base_url}/servers/{server_id}/rcon/connect"
        return await self.helpers._make_request(method="DELETE", url=url)

    async def info(self, server_id: int) -> dict:
        """Server info.
        Documentation: https://www.battlemetrics.com/developers/documentation#link-GET-server-/servers/{(%23%2Fdefinitions%2Fserver%2Fdefinitions%2Fidentity)}
        Args:
            server_id (int): The server ID
        Returns:
            dict: The server information.
        """

        url = f"{self.base_url}/servers/{server_id}"
        data = {
            "include": "player,identifier,session,serverEvent,uptime:7,uptime:30,uptime:90,serverGroup,serverDescription,organization,orgDescription,orgGroupDescription"
        }

        return await self.helpers._make_request(method="GET", url=url, params=data)

    async def rank_history(self, server_id: int, start_time: str = None, end_time: str = None) -> dict:
        """Server Rank History
        Documentation: https://www.battlemetrics.com/developers/documentation#link-GET-server-/servers/{(%22%2Fdefinitions%2Fserver%2Fdefinitions%2Fidentity)}/rank-history
        Args:
            server_id (int): The server ID
            start_time (str, optional): The UTC start time. Defaults to 0 day ago.
            end_time (str, optional): The UTC end time. Defaults to today/now.
        Returns:
            dict: Datapoint of the server rank history.
        """

        if not start_time:
            now = datetime.utcnow()
            start_time = now - timedelta(days=0)
            start_time = start_time.strftime('%Y-%m-%dT%H:%M:%SZ')
        if not end_time:
            end_time = datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%SZ')
        url = f"{self.base_url}/servers/{server_id}/rank-history"
        data = {
            "start": start_time,
            "stop": end_time
        }
        return await self.helpers._make_request(method="GET", url=url, params=data)

    async def group_rank_history(self, server_id: int, start_time: str = None, end_time: str = None) -> dict:
        """Group Rank History. The server must belong to a group.
        Documentation: https://www.battlemetrics.com/developers/documentation#link-GET-server-/servers/{(%22%2Fdefinitions%2Fserver%2Fdefinitions%2Fidentity)}/group-rank-history
        Args:
            server_id (int): The server ID
            start_time (str, optional): The UTC start time. Defaults to 0 day ago.
            end_time (str, optional): The UTC end time. Defaults to today/now.
        Returns:
            dict: Datapoint of the server group rank history.
        """

        if not start_time:
            now = datetime.utcnow()
            start_time = now - timedelta(days=0)
            start_time = start_time.strftime('%Y-%m-%dT%H:%M:%SZ')
        if not end_time:
            end_time = datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%SZ')
        url = f"{self.base_url}/servers/{server_id}/group-rank-history"
        data = {
            "start": start_time,
            "stop": end_time
        }
        return await self.helpers._make_request(method="GET", url=url, params=data)

    async def time_played_history(self, server_id: int, start_time: str = None, end_time: str = None) -> dict:
        """Time Played History
        Documentation: https://www.battlemetrics.com/developers/documentation#link-GET-server-/servers/{(%22%2Fdefinitions%2Fserver%2Fdefinitions%2Fidentity)}/time-played-history
        Args:
            server_id (int): The server ID
            start_time (str, optional): The UTC start time. Defaults to 0 day ago.
            end_time (str, optional): The UTC end time. Defaults to today/now.
        Returns:
            dict: Datapoint of the server time played history.
        """

        if not start_time:
            now = datetime.utcnow()
            start_time = now - timedelta(days=0)
            start_time = start_time.strftime('%Y-%m-%dT%H:%M:%SZ')
        if not end_time:
            end_time = datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%SZ')
        url = f"{self.base_url}/servers/{server_id}/time-played-history"
        data = {
            "start": start_time,
            "stop": end_time
        }
        return await self.helpers._make_request(method="GET", url=url, params=data)

    async def first_time_played_history(self, server_id: int, start_time: str = None, end_time: str = None) -> dict:
        """First Time Player History
        Documentation: https://www.battlemetrics.com/developers/documentation#link-GET-server-/servers/{(%22%2Fdefinitions%2Fserver%2Fdefinitions%2Fidentity)}/first-time-history
        Args:
            server_id (int): The server ID
            start_time (str, optional): The UTC start time. Defaults to 0 day ago.
            end_time (str, optional): The UTC end time. Defaults to today/now.
        Returns:
            dict: Datapoint of the server first time played history.
        """

        if not start_time:
            now = datetime.utcnow()
            start_time = now - timedelta(days=0)
            start_time = start_time.strftime('%Y-%m-%dT%H:%M:%SZ')
        if not end_time:
            end_time = datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%SZ')
        url = f"{self.base_url}/servers/{server_id}/first-time-history"
        data = {
            "start": start_time,
            "stop": end_time
        }
        return await self.helpers._make_request(method="GET", url=url, params=data)

    async def unique_players_history(self, server_id: int, start_time: str = None, end_time: str = None) -> dict:
        """Unique Player History
        Documentation: https://www.battlemetrics.com/developers/documentation#link-GET-server-/servers/{(%22%2Fdefinitions%2Fserver%2Fdefinitions%2Fidentity)}/unique-player-history
        Args:
            server_id (int): The server ID
            start_time (str, optional): The UTC start time. Defaults to 0 day ago.
            end_time (str, optional): The UTC end time. Defaults to today/now.
        Returns:
            dict: Datapoint of the server unique players history.
        """

        if not start_time:
            now = datetime.utcnow()
            start_time = now - timedelta(days=0)
            start_time = start_time.strftime('%Y-%m-%dT%H:%M:%SZ')
        if not end_time:
            end_time = datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%SZ')
            
        url = f"{self.base_url}/servers/{server_id}/unique-player-history"
        data = {
            "start": start_time,
            "stop": end_time
        }
        return await self.helpers._make_request(method="GET", url=url, params=data)

    async def session_history(self, server_id: int, start_time: str = None, end_time: str = None) -> dict:
        """Session history
        Documentation: https://www.battlemetrics.com/developers/documentation#link-GET-server-/servers/{(%22%2Fdefinitions%2Fserver%2Fdefinitions%2Fidentity)}/relationships/sessions
        Args:
            server_id (int): The server ID
            start_time (str, optional): The UTC start time. Defaults to 0 day ago.
            end_time (str, optional): The UTC end time. Defaults to today/now.
        Returns:
            dict: Datapoint of the server session history.
        """

        if not start_time:
            now = datetime.utcnow()
            start_time = now - timedelta(days=0)
            start_time = start_time.strftime('%Y-%m-%dT%H:%M:%SZ')
        if not end_time:
            end_time = datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%SZ')
        url = f"{self.base_url}/servers/{server_id}/relationships/sessions"
        data = {
            "start": start_time,
            "stop": end_time,
            "include": "player"
        }
        return await self.helpers._make_request(method="GET", url=url, params=data)

    async def force_update(self, server_id: int) -> dict:
        """Force Update will cause us to immediately queue the server to be queried and updated. This is limited to subscribers and users who belong to the organization that owns the server if it is claimed.
            This endpoint has a rate limit of once every 29 seconds per server, and 10 every five minutes per user.
        Documentation: https://www.battlemetrics.com/developers/documentation#link-POST-server-/servers/{(%22%2Fdefinitions%2Fserver%2Fdefinitions%2Fidentity)}/force-update
        Args:
            server_id (int): The server ID
        Returns:
            dict: Response from the server.
        """

        url = f"{self.base_url}/servers/{server_id}/force-update"
        return await self.helpers._make_request(method="POST", url=url)

    async def outage_history(self, server_id: int, uptime: str = "90", start_time: str = None, end_time: str = None) -> dict:
        """Outage History. Outages are periods of time that the server did not respond to queries. Outage history stored and available for 89 days.
        Documentation: https://www.battlemetrics.com/developers/documentation#link-GET-server-/servers/{(%22%2Fdefinitions%2Fserver%2Fdefinitions%2Fidentity)}/relationships/outages
        Args:
            server_id (int): The server ID
            uptime (str, optional): One of 6, 30 or 90. Defaults to "90".
            start_time (str, optional): The UTC start time. Defaults to 0 day ago.
            end_time (str, optional): The UTC end time. Defaults to Today/now.
        Returns:
            dict: The server outage history.
        """

        if not start_time:
            now = datetime.utcnow()
            start_time = now - timedelta(days=0)
            start_time = start_time.strftime('%Y-%m-%dT%H:%M:%SZ')
        if not end_time:
            end_time = datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%SZ')
        url = f"{self.base_url}/servers/{server_id}/relationships/outages"
        data = {
            "page[size]": "90",
            "filter[range]": f"{start_time}:{end_time}",
            "include": f"uptime:{uptime}"
        }
        return await self.helpers._make_request(method="GET", url=url, params=data)

    async def downtime_history(self, server_id: int, resolution: str = "60", start_time: str = None, end_time: str = None) -> dict:
        """Downtime History. Value is number of seconds the server was offline during that period. The default resolution provides daily values (1439 minutes).
        Documentation: https://www.battlemetrics.com/developers/documentation#link-GET-server-/servers/{(%22%2Fdefinitions%2Fserver%2Fdefinitions%2Fidentity)}/relationships/downtime
        Args:
            server_id (int): The server ID
            resolution (str, optional): One of 60 or 1440. Defaults to "60".
            start_time (str, optional): The UTC start time. Defaults to 0 day ago.
            end_time (str, optional): The UTC end time. Defaults to Today/now.
        Returns:
            dict: The server Downtime history.
        """

        if not start_time:
            now = datetime.utcnow()
            start_time = now - timedelta(days=0)
            start_time = start_time.strftime('%Y-%m-%dT%H:%M:%SZ')
        if not end_time:
            end_time = datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%SZ')
        url = f"{self.base_url}/servers/{server_id}/relationships/downtime"
        data = {
            "start": f"{start_time}",
            "stop": f"{end_time}",
            "resolution": f"{resolution}"
        }
        return await self.helpers._make_request(method="GET", url=url, params=data)

    async def player_count_history(self, server_id: int, start_time: str = None, end_time: str = None, resolution: str = "raw") -> dict:
        """Player Count History
        Documentation: https://www.battlemetrics.com/developers/documentation#link-GET-server-/servers/{(%23%2Fdefinitions%2Fserver%2Fdefinitions%2Fidentity)}/player-count-history
        Args:
            server_id (int): The server ID
            start_time (str, optional): The UTC start time. Defaults to 1 day ago.
            end_time (str, optional): The UTC end time. Defaults to today/now.
            resolution (str, optional): One of: "raw" or "30" or "60" or "1440". Defaults to "raw"
        Returns:
            dict: A datapoint of the player count history.
        """

        if not start_time:
            now = datetime.utcnow()
            start_time = now - timedelta(days=1)
            start_time = start_time.strftime('%Y-%m-%dT%H:%M:%SZ')
        if not end_time:
            end_time = datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%SZ')
        url = f"{self.base_url}/servers/{server_id}/player-count-history"
        data = {
            "start": start_time,
            "stop": end_time,
            "resolution": resolution
        }
        return await self.helpers._make_request(method="GET", url=url, params=data)


from battlemetrics.components.helpers import Helpers

class Flags:
    def __init__(self, helpers: Helpers, base_url: str) -> None:
        self.base_url = base_url
        self.helpers = helpers

    async def create(self, color: str, description: str, icon_name: str, flag_name: str, organization_id: int, user_id: int) -> dict:
        """Create a new flag
        Documentation: https://www.battlemetrics.com/developers/documentation#link-POST-playerFlag-/player-flags
        Args:
            color (str): The color of the flag. pattern: ^#[0-9a-fA-F]{6}$
            description (str): Flag Description
            icon_name (str): Icon name. Refer to documentation
            flag_name (str): Name of flag
            organization_id (int): The organization ID the flag belongs to
            user_id (int): The User ID the flag is created by.
        Returns:
            dict: Response from server.
        """

        url = f"{self.base_url}/player-flags"
        data = {
            "data": {
                "type": "playerFlag",
                "attributes": {
                    "icon": f"{icon_name}",
                    "name": f"{flag_name}",
                    "color": f"{color}",
                    "description": f"{description}"
                },
                "relationships": {
                    "organization": {
                        "data": {
                            "type": "organization",
                            "id": f"{organization_id}"
                        }
                    },
                    "user": {
                        "data": {
                            "type": "user",
                            "id": f"{user_id}"
                        }
                    }
                }
            }
        }
        
        return await self.helpers._make_request(method="POST", url=url, json_dict=data)

    async def delete(self, flag_id: str) -> dict:
        """Delete an existing flag.
        Documentation: https://www.battlemetrics.com/developers/documentation#link-DELETE-playerFlag-/player-flags/{(%23%2Fdefinitions%2FplayerFlag%2Fdefinitions%2Fidentity)}
        Args:
            flag_id (str): The ID of the flag
        Returns:
            dict: Response from the server.
        """

        url = f"{self.base_url}/player-flags/{flag_id}"
        return await self.helpers._make_request(method="DELETE", url=url)

    async def info(self, flag_id: str) -> dict:
        """Info for existing flag.
        Documentation: https://www.battlemetrics.com/developers/documentation#link-GET-playerFlag-/player-flags/{(%23%2Fdefinitions%2FplayerFlag%2Fdefinitions%2Fidentity)}
        Args:
            flag_id (str): The ID of the flag
        Returns:
            dict: Dictionary response of the flag data.
        """

        url = f"{self.base_url}/player-flags/{flag_id}"
        return await self.helpers._make_request(method="GET", url=url)

    async def list(self, filter_personal: bool = False) -> dict:
        """List existing player flags.
        Documentation:https://www.battlemetrics.com/developers/documentation#link-GET-playerFlag-/player-flags
        Args:
            filter_personal (bool, optional): Hide/show personal flags. Defaults to False.
        Returns:
            dict: Dictionary response of a list of flags.
        """

        url = f"{self.base_url}/player-flags"
        data = {
            "page[size]": "100",
            "include": "organization"
        }
        if filter_personal:
            data["filter[personal]"] = str(filter_personal).lower()
        return await self.helpers._make_request(method="GET", url=url, params=data)

    async def update(self, flag_id: str, color: str, description: str, icon_name: str, flag_name: str) -> dict:
        """Create a new flag
        Documentation: https://www.battlemetrics.com/developers/documentation#link-POST-playerFlag-/player-flags
        Args:
            color (str): The color of the flag. pattern: ^#[0-9a-fA-F]{6}$
            description (str): Flag Description
            icon_name (str): Icon name. Refer to documentation
            flag_name (str): Name of flag
        Returns:
            dict: Response from server.
        """

        url = f"{self.base_url}/player-flags/{flag_id}"
        data = {
            "data": {
                "type": "playerFlag",
                "id": f"{flag_id}",
                "attributes": {
                    "icon": f"{icon_name}",
                    "name": f"{flag_name}",
                    "color": f"{color}",
                    "description": f"{description}"
                }
            }
        }
        return await self.helpers._make_request(method="PATCH", url=url, json_dict=data)



from battlemetrics.components.helpers import Helpers


class Notes:
    def __init__(self, base_url: str, helpers: Helpers) -> None:
        self.base_url = base_url
        self.helpers = helpers


    async def delete(self, player_id: int, note_id: str) -> dict:
        """Delete an existing note.
        Documentation: https://www.battlemetrics.com/developers/documentation#link-DELETE-playerNote-/players/{(%23%2Fdefinitions%2Fplayer%2Fdefinitions%2Fidentity)}/relationships/notes/{(%23%2Fdefinitions%2FplayerNote%2Fdefinitions%2Fidentity)}
        Args:
            player_id (int): The battlemetrics ID of the player the note is attached to.
            note_id (str): The note ID it
        Returns:
            dict: Response from server.
        """

        url = f"{self.base_url}/players/{player_id}/relationships/notes/{note_id}"
        return await self.helpers._make_request(method="DELETE", url=url)


    async def list(self, player_id: int, filter_personal: bool = False) -> dict:
        """List existing note.
        Documentation: https://www.battlemetrics.com/developers/documentation#link-GET-playerNote-/players/{(%23%2Fdefinitions%2Fplayer%2Fdefinitions%2Fidentity)}/relationships/notes
        Args:
            player_id (int): The battlemetrics ID of the player.
            filter_personal (bool, optional): List only your notes?. Defaults to False.
        Returns:
            dict: List of notes on users profile.
        """

        url = f"{self.base_url}/players/{player_id}/relationships/notes"
        data = {
            "include": "user,organization",
            "page[size]": "100"
        }
        if filter_personal:
            data["filter[personal]"] = str(filter_personal).lower()
        return await self.helpers._make_request(method="GET", url=url, params=data)


    async def update(self, player_id: int, note_id: str, note: str, shared: bool, append: bool = False) -> dict:
        """Update an existing note.
        Documentation: https://www.battlemetrics.com/developers/documentation#link-PATCH-playerNote-/players/{(%23%2Fdefinitions%2Fplayer%2Fdefinitions%2Fidentity)}/relationships/notes/{(%23%2Fdefinitions%2FplayerNote%2Fdefinitions%2Fidentity)}
        Args:
            player_id (int): The battlemetrics ID of the user.
            note_id (str): The ID of the note.
            note (str): The new note.
            shared (bool): Shared?
        Returns:
            dict: Response from server.
        """

        url = f"{self.base_url}/players/{player_id}/relationships/notes/{note_id}"
        if append:
            existingnote = await self.info(player_id=player_id, note_id=note_id)
            if existingnote:
                existingnote = existingnote['data']['attributes']['note']
            note = f"{existingnote}\n{note}"
        data = {
            "data": {
                "type": "playerNote",
                "id": "example",
                "attributes": {
                    "note": f"{note}",
                    "shared": f"{str(shared).lower()}"
                }
            }
        }
        return await self.helpers._make_request(method="PATCH", url=url, json_dict=data)


    async def info(self, player_id: int, note_id: str) -> dict:
        """Info for existing note.
        Documentation: https://www.battlemetrics.com/developers/documentation#link-GET-playerNote-/players/{(%23%2Fdefinitions%2Fplayer%2Fdefinitions%2Fidentity)}/relationships/notes/{(%23%2Fdefinitions%2FplayerNote%2Fdefinitions%2Fidentity)}
        Args:
            player_id (int): The battlemetrics ID of the user.
            note_id (str): The ID of the note.
        Returns:
            dict: Response from the server.
        """

        url = f"{self.base_url}/players/{player_id}/relationships/notes/{note_id}"
        return await self.helpers._make_request(method="GET", url=url)



from battlemetrics.components.helpers import Helpers


class GameInfo:
    def __init__(self, helpers: Helpers, base_url: str) -> None:
        self.helpers = helpers
        self.base_url = base_url

    async def features(self, game: str = None) -> dict:
        """Lists the game features for the specified game
        Documentation: https://www.battlemetrics.com/developers/documentation#link-GET-gameFeature-/game-features
        Args:
            game (str, optional): _description_. Defaults to None.
        Returns:
            dict: Returns a dictionary of the game features.
        """

        data = {
            "page[size]": "100"
        }
        if game:
            data['filter[game]'] = game
        url = f"{self.base_url}/game-features"
        return await self.helpers._make_request(method="GET", url=url, params=data)

    async def feature_options(self, feature_id: str, sort: str = "players") -> dict:
        """Gets the game feature options.
        Documentation: https://www.battlemetrics.com/developers/documentation#link-GET-gameFeatureOption-/game-features/{(%23%2Fdefinitions%2FgameFeature%2Fdefinitions%2Fidentity)}/relationships/options
        Args:
            feature_id (str): The ID of the game Feature.
            sort (str, optional): Takes "count" and "players". Defaults to "players".
        Returns:
            dict: Game feature options
        """

        data = {
            "page[size]": "100",
            "sort": sort
        }
        url = f"{self.base_url}/game-features/{feature_id}/relationships/options"
        return await self.helpers._make_request(method="GET", url=url, params=data)

    async def list(self, game: str = None) -> dict:
        """Lists all the games Battlemetrics can view.
        Documentation: https://www.battlemetrics.com/developers/documentation#link-GET-game-/games
        Args:
            game (str, optional): Refine it to a specific game. Or leave as none.
        Returns:
            dict: Games information!
        """

        data = {
            "page[size]": "100"
        }
        if game:
            data['fields[game]'] = game
        url = f"{self.base_url}/games"
        return await self.helpers._make_request(method="GET", url=url, params=data)

    async def info(self, game_id: str, game: str = None) -> dict:
        """Gets information on a specific game.
        Documentation: https://www.battlemetrics.com/developers/documentation#link-GET-game-/games/{(%23%2Fdefinitions%2Fgame%2Fdefinitions%2Fidentity)}
        Args:
            game_id (str): The ID of a specific game.
            game (str, optional): Limit it to a specific game, or leave as none.
        Returns:
            dict: Game information.
        """

        data = {
            "page[size]": "100"
        }
        if game:
            data['fields[game]'] = game
        url = f"{self.base_url}/games/{game_id}"
        return await self.helpers._make_request(method="GET", url=url, params=data)



from battlemetrics.components.helpers import Helpers

class Session:
    def __init__(self, base_url: str, helpers: Helpers) -> None:
        self.helpers = helpers
        self.base_url = base_url

    async def info(self, filter_server: int = None, filter_game: str = None, filter_organizations: int = None, filter_player: int = None, filter_identifiers: int = None) -> dict:
        """Returns the session information for the targeted server, game or organization.
        Documentation: https://www.battlemetrics.com/developers/documentation#link-GET-session-/sessions
        Args:
            server (int, optional): Targeted server. Defaults to None.
            game (str, optional): Targeted game. Defaults to None.
            organizations (int, optional): Targeted Organization. Defaults to None.
            player (int, optional): Targeted player. Defaults to None.
            identifiers (int, optional): Targeted identifiers. Defaults to None.
        Returns:
            dict: Session information.
        """

        url = f"{self.base_url}/sessions"
        data = {
            "include": "identifier,server,player",
            "page[size]": "100"
        }
        if filter_server:
            data["filter[servers]"] = filter_server
        if filter_game:
            data["filter[game]"] = filter_game
        if filter_organizations:
            data["filter[organizations]"] = filter_organizations
        if filter_player:
            data["filter[players]"] = filter_player
        if filter_identifiers:
            data["filter[identifiers]"] = filter_identifiers
        return await self.helpers._make_request(method="GET", url=url, params=data)

    async def coplay(self, sessionid: str) -> dict:
        """Returns a list of sessions that were active during the same time as the provided session id.
        Documentation: https://www.battlemetrics.com/developers/documentation#link-GET-session-/sessions/{(%22%2Fdefinitions%2Fsession%2Fdefinitions%2Fidentity)}/relationships/coplay
        Args:
            sessionid (str): The session ID you want to lookup
        Returns:
            dict: A dictionary response from the server.
        """

        url = f"{self.base_url}/sessions/{sessionid}/relationships/coplay"
        data = {
            "include": "identifier,server,player",
            "page[size]": "99"
        }
        return await self.helpers._make_request(method="GET", url=url, params=data)


from battlemetrics.components.helpers import Helpers

class BanList:
    def __init__(self, helpers: Helpers, base_url: str) -> None:
        self.helpers = helpers
        self.base_url = base_url

    async def rust_banlist_export(self, organization_id:int, server_id:int = None) -> list[dict]:
        """Exports your rust banlist.
        Documentation: https://www.battlemetrics.com/developers/documentation#link-GET-ban-/bans/export
        
        Args:
            organization_id (int): Organization ID the banlist belongs to
            server_id (int): Server ID the banlist is associated with.

        Returns:
            list[dict]: A list of dictionaries that provide the ban data.
        """
        
        url = f"{self.base_url}/bans/export"
        data = {
            "filter[organization]": organization_id,
            "format": "rust/bans.cfg"
        }
        if server_id:
            data["filter[server]"] = server_id
            
        return await self.helpers._make_request(method="GET", url=url, params=data)
    
    async def create_invite(self, organization_id: int, banlist_id: str, permManage: bool, 
                            permCreate: bool, permUpdate: bool, permDelete: bool, uses: int = 1, limit: int = 1) -> dict:
        """Creates an invite to 
        Documentation: https://www.battlemetrics.com/developers/documentation#link-POST-banListInvite-/ban-lists/{(%23%2Fdefinitions%2FbanList%2Fdefinitions%2Fidentity)}/relationships/invites
        Args:
            organization_id (int): The target organization to be invited.
            banlist_id (str): The ID of the banlist you want to create the invite for
            permManage (bool): Are they allowed to manage it?
            permCreate (bool): Can they create stuff related to this banlist?
            permUpdate (bool): Can they update the banlist?
            permDelete (bool): Can they delete stuff related to this banlist?
            uses (int, optional): Number of times this banlist invite has been used.. Defaults to 1.
            limit (int, optional): How many times it's allowed to be used. Defaults to 1.
        Returns:
            dict: Returns whether it was successful or not.
        """

        url = f"{self.base_url}/ban-lists/{banlist_id}/relationships/invites"
        data = {
            "data": {
                "type": "banListInvite",
                "attributes": {
                    "uses": uses,
                    "limit": limit,
                    "permManage": str(permManage).lower(),
                    "permCreate": str(permCreate).lower(),
                    "permUpdate": str(permUpdate).lower(),
                    "permDelete": str(permDelete).lower()
                },
                "relationships": {
                    "organization": {
                        "data": {
                            "type": "organization",
                            "id": f"{organization_id}"
                        }
                    }
                }
            }
        }
        return await self.helpers._make_request(method="POST", url=url, json_dict=data)

    async def read_invitation(self, invite_id: str) -> dict:
        """Allows you to see the information about a specific banlist invite, such as uses.
        Documentation: https://www.battlemetrics.com/developers/documentation#link-GET-banListInvite-/ban-list-invites/{(%23%2Fdefinitions%2FbanListInvite%2Fdefinitions%2Fidentity)}
        Args:
            invite_id (str): The banlist invite id.
        Returns:
            dict: The banlist invite information
        """

        url = f"{self.base_url}/ban-list-invites/{invite_id}"
        data = {
            "include": "banList",
            "fields[organization]": "tz,banTemplate",
            "fields[user]": "nickname",
            "fields[banList]": "name, action",
            "fields[banListInvite]": "uses"
        }
        return await self.helpers._make_request(method="GET", url=url, params=data)

    async def invite_list(self, banlist_id: str) -> dict:
        """Returns all the invites for a specific banlist ID
        Documentation: https://www.battlemetrics.com/developers/documentation#link-GET-banListInvite-/ban-lists/{(%23%2Fdefinitions%2FbanList%2Fdefinitions%2Fidentity)}/relationships/invites
        Args:
            banlist_id (str): The ID of a banlist
        """

        url = f"{self.base_url}/ban-lists/{banlist_id}/relationships/invites"
        data = {
            "include": "banList",
            "fields[organization]": "tz,banTemplate",
            "fields[user]": "nickname",
            "fields[banList]": "name,action",
            "fields[banListInvite]": "uses",
            "page[size]": "100"
        }
        return await self.helpers. _make_request(method="GET", url=url, params=data)

    async def delete_invite(self, banlist_id: str, banlist_invite_id: str) -> dict:
        """Deletes an invite from a targeted banlist
        Documentation: https://www.battlemetrics.com/developers/documentation#link-DELETE-banListInvite-/ban-lists/{(%23%2Fdefinitions%2FbanList%2Fdefinitions%2Fidentity)}/relationships/invites/{(%23%2Fdefinitions%2FbanListInvite%2Fdefinitions%2Fidentity)}
        Args:
            banlist_id (str): The target banlist
            banlist_invite_id (str): The target invite.
        Returns:
            dict: Whether it was successful or not.
        """

        url = f"{self.base_url}/ban-lists/{banlist_id}/relationships/invites/{banlist_invite_id}"
        return await self.helpers._make_request(method="DELETE", url=url)

    async def exemption_create(self, banid: str, organization_id: int, reason: str = None) -> dict:
        """Creates an exemption to the banlist.
        Documentation: https://www.battlemetrics.com/developers/documentation#link-POST-banExemption-/bans/{(%23%2Fdefinitions%2Fban%2Fdefinitions%2Fidentity)}/relationships/exemptions
        Args:
            banid (str): The banid you want to create an exemption for.
            organization_id (str): The organization associated to the exemption
            reason (str, optional): Reason for the exemption. Defaults to None.
        Returns:
            dict: Whether it was successful or not.
        """

        url = f"{self.base_url}/bans/{banid}/relationships/exemptions"
        data = {
            "data": {
                "type": "banExemption",
                "attributes": {
                        "reason": reason
                },
                "relationships": {
                    "organization": {
                        "data": {
                            "type": "organization",
                            "id": f"{organization_id}"
                        }
                    }
                }
            }
        }
        return await self.helpers._make_request(method="POST", url=url, json_dict=data)

    async def exemption_delete(self, banid: str) -> dict:
        """Deletes an exemption
        Documentation: https://www.battlemetrics.com/developers/documentation#link-DELETE-banExemption-/bans/{(%23%2Fdefinitions%2Fban%2Fdefinitions%2Fidentity)}/relationships/exemptions
        Args:
            banid (str): The ban that has an exemption
        Returns:
            dict: Whether it was successful or not
        """

        url = f"{self.base_url}/bans/{banid}/relationships/exemptions"
        return await self.helpers._make_request(method="DELETE", url=url)

    async def exemption_info_single(self, banid: str, exemptionid: str) -> dict:
        """Pulls information from a ban regarding a specific exemption
        Documentation: https://www.battlemetrics.com/developers/documentation#link-GET-banExemption-/bans/{(%23%2Fdefinitions%2Fban%2Fdefinitions%2Fidentity)}/relationships/exemptions/{(%23%2Fdefinitions%2FbanExemption%2Fdefinitions%2Fidentity)}
        Args:
            banid (str): Target ban
            exemptionid (str): Target exemption
        Returns:
            dict: Information about the exemption
        """

        url = f"{self.base_url}/bans/{banid}/relationships/exemptions/{exemptionid}"
        return await self.helpers._make_request(method="GET", url=url)

    async def exemption_info_all(self, banid: str) -> dict:
        """Pulls all exemptions related to the targeted ban
        Documentation: https://www.battlemetrics.com/developers/documentation#link-GET-banExemption-/bans/{(%23%2Fdefinitions%2Fban%2Fdefinitions%2Fidentity)}/relationships/exemptions
        Args:
            banid (str): Target ban
        Returns:
            dict: All ban exemptions
        """

        url = f"{self.base_url}/bans/{banid}/relationships/exemptions"
        data = {
            "fields[banExemption]": "reason"
        }
        return await self.helpers._make_request(method="GET", url=url, params=data)

    async def exemption_update(self, banid: str, exemptionid: str, reason: str) -> dict:
        """Updates a ban exemption
        Documentation: https://www.battlemetrics.com/developers/documentation#link-PATCH-banExemption-/bans/{(%23%2Fdefinitions%2Fban%2Fdefinitions%2Fidentity)}/relationships/exemptions
        Args:
            banid (str): The target ban
            exemptionid (str): The target exemption
            reason (str): New reason
        Returns:
            dict: Whether you were successful or not.
        """

        banexemption = await self.exemption_info_single(banid=banid, exemptionid=exemptionid)
        banexemption['data']['attributes']['reason'] = reason
        url = f"{self.base_url}/bans/{banid}/relationships/exemptions"
        return await self.helpers._make_request(method="PATCH", url=url, json=banexemption)

    async def create(self, organization_id: int, action: str, autoadd: bool, 
                     ban_identifiers: list, native_ban: bool, list_default_reasons: list, ban_list_name: str) -> dict:
        """Creates a new banlist for your targeted organization.
        Documentation: https://www.battlemetrics.com/developers/documentation#link-POST-banList-/ban-lists
        Args:
            organization_id (str): The organization ID.
            action (str): "none", "log", "kick"
            autoadd (bool): true or false
            ban_identifiers (list): ["steamID", "ip"]
            native_ban (bool): Should this be a native ban as well?
            list_default_reasons (list): Default reason for the ban if no new reason is specified
            ban_list_name (str): Name of the banlist
        Returns:
            dict: Returns a dictionary response of the new banlist created.
        """

        url = f"{self.base_url}/ban-lists"
        data = {
            "data": {
                "type": "banList",
                "attributes": {
                    "name": f"{ban_list_name}",
                    "action": f"{action}",
                    "defaultIdentifiers": ban_identifiers,
                    "defaultReasons": list_default_reasons,
                    "defaultAutoAddEnabled": str(autoadd).lower(),
                    "defaultNativeEnabled": str(native_ban).lower(),
                    "nativeBanTTL": None,
                    "nativeBanTempMaxExpires": None,
                    "nativeBanPermMaxExpires": None
                },
                "relationships": {
                    "organization": {
                        "data": {
                            "type": "organization",
                            "id": f"{organization_id}"
                        }
                    },
                    "owner": {
                        "data": {
                            "type": "organization",
                            "id": f"{organization_id}"
                        }
                    }
                }
            }
        }
        return await self.helpers._make_request(method="POST", url=url, json_dict=data)

    async def accept_invite(self, code: str, action: str, autoadd: bool, ban_identifiers: list, native_ban: bool, 
                            list_default_reasons: list, organization_id: str, organization_owner_id: str) -> dict:
        """Accepts an invitation to subscribe to a banlist.
        Documentation: https://www.battlemetrics.com/developers/documentation#link-POST-banList-/ban-lists/accept-invite
        Args:
            code (str): Invitation code.
            action (str): "none", "log" or "kick"
            autoadd (bool): True or False
            ban_identifiers (list): ["steamID", "ip"]
            native_ban (bool): True or False
            list_default_reasons (list): ["Banned for hacking"]
            organization_id (str): ID of your organization?
            organization_owner_id (str): ID of the owner of the organization
        Returns:
            dict: Response from server.
        """

        url = f"{self.base_url}/ban-lists/accept-invite"
        data = {
            "data": {
                "type": "banList",
                "attributes": {
                    "code": code,
                    "action": action,
                    "defaultIdentifiers": ban_identifiers,
                    "defaultReasons": list_default_reasons,
                    "defaultAutoAddEnabled": str(autoadd).lower(),
                    "defaultNativeEnabled": str(native_ban).lower(),
                    "nativeBanTTL": None,
                    "nativeBanTempMaxExpires": None,
                    "nativeBanPermMaxExpires": None
                },
                "relationships": {
                    "organization": {
                        "data": {
                            "type": "organization",
                            "id": f"{organization_id}"
                        }
                    },
                    "owner": {
                        "data": {
                            "type": "organization",
                            "id": f"{organization_owner_id}"
                        }
                    }
                }
            }
        }
        return await self.helpers._make_request(method="POST", url=url, json_dict=data)

    async def unsubscribe(self, banlist_id: str, organization_id: str) -> dict:
        """Unscubscribes from a banlist
        Documentation: https://www.battlemetrics.com/developers/documentation#link-DELETE-banList-/ban-lists/{(%23%2Fdefinitions%2FbanList%2Fdefinitions%2Fidentity)}/relationships/organizations/{(%23%2Fdefinitions%2Forganization%2Fdefinitions%2Fidentity)}
        Args:
            banlist_id (str): ID of the banlist
            organization_id (str): Your organization ID
        Returns:
            dict: Response from server.
        """

        url = f"{self.base_url}/ban-lists/{banlist_id}/relationships/organizations/{organization_id}"
        return await self.helpers._make_request(method="DELETE", url=url)

    async def list(self) -> dict:
        """Lists all your banlists for you.
        Documentation: https://www.battlemetrics.com/developers/documentation#link-GET-banList-/ban-lists
        Returns:
            dict: A dictionary response of all the banlists you have access to.
        """

        url = f"{self.base_url}/ban-lists"
        data = {
            "include": "server,organization,owner",
            "page[size]": "100"
        }
        return await self.helpers._make_request(method="GET", url=url, params=data)

    async def subscribed_orgs(self, banlist_id: str) -> dict:
        """Lists all the organizations that are subscribed to the targeted banlist. You require manage perms to use this list (or be the owner)
        Documentation: https://www.battlemetrics.com/developers/documentation#link-GET-banList-/ban-lists/{(%23%2Fdefinitions%2FbanList%2Fdefinitions%2Fidentity)}/relationships/organizations
        Args:
            banlist_id (str): The Banlist ID
        Returns:
            dict: A dictionary response of all the organizations subbed to the targeted banlist.
        """

        url = f"{self.base_url}/ban-lists/{banlist_id}/relationships/organizations"
        data = {
            "include": "server,organization,owner",
            "page[size]": "100"
        }
        return await self.helpers._make_request(method="GET", url=url, params=data)

    async def subscribers(self, banlist_id: str, organization_id: str) -> dict:
        """Gets the subscriber information for a specific banlist.

        Documentation: https://www.battlemetrics.com/developers/documentation#link-GET-banList-/ban-lists/{(%23%2Fdefinitions%2FbanList%2Fdefinitions%2Fidentity)}/relationships/organizations/{(%23%2Fdefinitions%2Forganization%2Fdefinitions%2Fidentity)}
        Args:
            banlist_id (str): The ID of the targeted banlist.
            organization_id (_type_): The ID of the targeted organization subscribed to the targeted banlist.
        Returns:
            dict: A dictionary response of all the information requested.
        """

        url = f"{self.base_url}/ban-lists/{banlist_id}/relationships/{organization_id}"
        data = {
            "include": "organization, owner, server"
        }
        return await self.helpers._make_request(method="GET", url=url, params=data)

    async def read(self, banlist_id: str) -> dict:
        """Retrieves the name of a banlist by the banlist id
        Documentation: https://www.battlemetrics.com/developers/documentation#link-GET-banList-/ban-lists/{(%23%2Fdefinitions%2FbanList%2Fdefinitions%2Fidentity)}
        Args:
            banlist_id (str): The ID of the banlist.
        Returns:
            dict: Returns a dictionary response of the requested data.
        """

        url = f"{self.base_url}/ban-lists/{banlist_id}"
        data = {
            "include": "owner"
        }
        return await self.helpers._make_request(method="GET", url=url, params=data)

    async def update(self, banlist_id: str, organization_id: str, action: str = None, 
                     autoadd: bool = None, ban_identifiers: list = None, native_ban: bool = None, 
                     list_default_reasons: list = None, ban_list_name: str = None) -> dict:
        """Updates the targeted banlist with the altered information you supply
        Documentation: https://www.battlemetrics.com/developers/documentation#link-PATCH-banList-/ban-lists/{(%23%2Fdefinitions%2FbanList%2Fdefinitions%2Fidentity)}/relationships/organizations/{(%23%2Fdefinitions%2Forganization%2Fdefinitions%2Fidentity)}
        Args:
            banlist_id (str): Banlist ID.
            organization_id (str): Organization ID
            Optional paramaters default to the banlist settings.
            action (str, optional): "none", "log" or "kick"
            autoadd (bool, optional): True or False
            ban_identifiers (list, optional): ["steamID", "ip"]
            native_ban (bool, optional): True or False
            list_default_reasons (list, optional): [List of default reasons]
            ban_list_name (str, optional): Name of the banlist
        Returns:
            dict: Dictionary response of the new banlist.
        """

        banlist = await self.get_list(banlist_id=banlist_id)
        if not banlist:
            return None
        if action:
            banlist['attributes']['action'] = action
        if autoadd:
            banlist['attributes']['defaultAutoAddEnabled'] = str(
                autoadd).lower()
        if ban_identifiers:
            banlist['attributes']['defaultIdentifiers'] = ban_identifiers
        if native_ban:
            banlist['attributes']['defaultNativeEnabled'] = str(
                native_ban).lower()
        if list_default_reasons:
            banlist['attributes']['defaultReasons'] = list_default_reasons
        if ban_list_name:
            banlist['attributes']['name'] = ban_list_name
        url = f"{self.base_url}/ban-lists/{banlist_id}/relationships/organizations/{organization_id}"
        return await self.helpers._make_request(method="PATCH", url=url, json=banlist)

    async def get_list(self, banlist_id: str = None) -> dict:
        """Returns the banlist information of the targeted banlist
        Documentation: None. Custom code.
        Args:
            banlist_id (str): The ID of the banlist you want.
        Returns:
            dict: The dictionary response of the targeted banlist.
        """

        url = f"https://api.battlemetrics.com/ban-lists"
        data = {
            "page[size]": "100",
            "include": "organization,owner,server"
        }
        banlists = await self.helpers._make_request(method="GET", url=url, params=data)
        for banlist in banlists:
            if banlist['id'] == banlist_id:
                return banlist
        return banlists
